#---------------------------------------------------------------------
# 1. Check the data with s2dverification map plotting functions
# 2. Read associated data of another data set matching the dates of the first one
# 3. How to use paramters `xxx_reorder`, `xxx_across`, `merge_across_dims`, `split_multiselected_dims`
#---------------------------------------------------------------------
library(startR)
library(s2dverification)

path <- '/esarchive/exp/ecearth/a0lg/monthly_mean/$var$_*/'
repos <- paste0(path, '$var$_*_S$sdate$_$member$_$chunk$.nc')
header <- Start(dat = repos,
                var = 'tas',
                member = indices(1:5),
                sdate = indices(1:21 * 2),
                chunk = 'all',
                ftime = indices(1:3),
                lat = 'all',
                lon = 'all',
                lat_reorder = Sort(),
                lon_reorder = CircularSort(0, 360),
                chunk_depends = 'sdate',
                ftime_across = 'chunk',
                merge_across_dims = TRUE,
                split_multiselected_dims = TRUE,
                synonims = list(ftime = 'time',
                                lat = c('lat', 'latitude'),
                                lon = c('lon', 'longitude')),
                return_vars = list(lon = NULL, 
                                   lat = NULL, 
                                   time = 'sdate')
               )

# ------ Check exp data ------
 attr(header, 'Dimensions')
#   dat    var member  sdate  ftime    lat    lon 
#     1      1      5     21      3    256    512 
# ----------------------------

lon <- attr(header, 'Variables')$common$lon
lat <- attr(header, 'Variables')$common$lat

data <- eval(header)  #retrieve data


# check 5 members
PlotLayout(PlotEquiMap, 
           c('lon', 'lat'),
           Subset(data, c('dat', 'var', 'sdate', 'ftime'), list(1, 1, 1, 1)),
           lon, 
           lat, 
           fill = FALSE
          )

# check 3 ftimes
PlotLayout(PlotEquiMap, 
           c('lon', 'lat'),
           Subset(data, c('dat', 'var', 'sdate', 'member'), list(1, 1, 1, 1)),
           lon, 
           lat, 
           fill = FALSE
          )

# check 21 sdates
PlotLayout(PlotStereoMap, 
           c('lon', 'lat'),
           Subset(data, c('dat', 'var', 'ftime', 'member'), list(1, 1, 1, 1)),
           lon, 
           lat, 
           fill = FALSE
          )


#-----------------------------
# Associated obs data
#-----------------------------
dates <- attr(header, 'Variables')$common$time  #use date attributes in exp data
#> dim(dates)
#sdate ftime 
#   21     3 

path <- '/esarchive/recon/ecmwf/erainterim/monthly_mean/$var$_*/'
repos <- paste0(path, '$var$_$date$.nc')
data_obs <- Start(dat = repos,
              var = 'tas',
              date = unique(format(dates, '%Y%m')),
              ftime = values(dates),
              lat = 'all',
              lon = 'all',
              lat_reorder = Sort(),
              lon_reorder = CircularSort(0, 360),
              # ftime continues along date 
              ftime_across = 'date',
              # merge ftime and date into one dim
              merge_across_dims = TRUE,
              # separate ftime dimension, which is [sdate = 21, ftime = 3]
              split_multiselected_dims = TRUE,
              synonims = list(ftime = 'time',
                              lat = c('lat', 'latitude'),
                              lon = c('lon', 'longitude')),
              return_vars = list(lon = NULL, 
                                 lat = NULL,
                                 ftime = 'date') 
             )
# ------ Check obs data ------
 attr(data_obs, 'Dimensions')
#  dat   var sdate ftime   lat   lon 
#    1     1    21     3   256   512 
# ----------------------------

# retrieve obs data into workstation
data_obs <- eval(data_obs)

# check 3 ftimes
PlotLayout(PlotEquiMap, 
           c('lon', 'lat'),
           Subset(data_obs, c('dat', 'var', 'sdate'), list(1, 1, 1)),
           attr(data_obs, 'Variables')$common$lon, 
           attr(data_obs, 'Variables')$common$lat, 
           fill = FALSE
          )

# check 2 sdates
PlotLayout(PlotEquiMap, 
           c('lon', 'lat'),
           Subset(data_obs, c('dat', 'var', 'sdate', 'ftime'), list(1, 1, 1:2, 1)),
           attr(data_obs, 'Variables')$common$lon, 
           attr(data_obs, 'Variables')$common$lat,  
           fill = FALSE
          )

